#ifndef CVECTOR_H_
#define CVECTOR_H_

#include <math.h>
#include <stdio.h>
///class For Handling Vectors used in Game

class Vector
{
	friend Vector operator *(float f, const Vector &vector);
	friend Vector operator -(const Vector &vector);
public:
	Vector Cross(const Vector &vector1);//, const Vector &vector2);
	float Dot(const Vector &vector1);//, const Vector &vector2);

	///Empty Constructor
	Vector() { x=y=z=0; }
	///Constructor that also sets the x y z values
	Vector(float x, float y, float z);
	Vector(const Vector &v) { *this = v; }
	///Empty Deconstructor
	~Vector() {}
	///Set the x y z of the vector
	void set(float x, float y, float z);

	/// == Operator
	///@param vector - vector to compare to
	bool operator ==(const Vector &vector) const;

	/// != Operator
	///@param vector - vector to compare to
	bool operator !=(const Vector &vector) const;

	void operator =(const Vector &v);

	/// less than Operator
	///@param vector - vector to compare to
	bool operator <(const Vector &vector) const;

	/// Greater than Operator
	///@param vector - vector to compare to
 	bool operator >(const Vector &vector) const;

	/// += Operator
	///@param vector - vector to add to
	Vector &operator +=(const Vector &vector);

	///accessor []
	///@param row accessor for array
	//float *operator [](int row);

	/// -= Operator
	///@param vector - vector to take away from
	Vector &operator -=(const Vector &vector);

	/// /= operator
	///@param f - float to /= to the vector
	Vector &operator /=(float f);

	/// *= operator
	///@param f - float to *= to te vector
	Vector &operator *=(float f);

	/// - operator
	///@param vector - Vector to take away from this vector
	Vector operator -(const Vector &vector) const;
	/// = operator
	///@param f - float value to make make the vector
	void operator =(const float f);
	///+ operator
	///@param
	Vector operator +(const Vector &vector) const;

	/// devide operator
	/// @param f - number to devide vector by
	Vector operator /(float f) const;

	/// devide operator
	/// @param v - number to devide vector by
	Vector operator /(Vector v) const;

	/// * Operator
	///@param f - number to multiply vector by
	Vector operator *(float f) const;

	///* operator
	///@param v - Vector to multipy by
	Vector operator *(Vector v) const;

	///makes x y z = 0
	void Normalize();
	/// gets total length of vector
	float GetLegnth();
	/// gets the distance of the vector
	///@param v - vector
	//float GetDistance(const Vector &v);
	float Magnitude() const;
	float MagnitudeSq() const;
	Vector Inverse() const;
	///Finds the inverse square root of a number
	///@param x - number to find inverse square root of
	float InvSqrt(float x);
	///gets x y z numbers from string
	///@param in - Input string holding x y z numbers
	//void set(char *in);


	///Vectors x number
	float x;
	///Vectors y number
	float y;
	///Vectors z number
	float z;



};

inline void Vector::operator =(const float f)
{
	x = y = z = f;
};

inline Vector operator *(float f, const Vector &vector)
{
	return Vector(vector.x * f, vector.y * f, vector.z * f);
};

inline void Vector::operator =(const Vector &v)
{
	x = v.x;
	y = v.y;
	z = v.z;
};

inline Vector operator -(const Vector &vector)
{
	return Vector(-vector.x, -vector.y, -vector.z);
};

inline Vector Vector::Cross(const Vector &vector1)//, const Vector &vector2)
{
	return Vector((vector1.y * z) - (vector1.z * y),
		      (vector1.z * x) - (vector1.x * z),
		      (vector1.x * y) - (vector1.y * x));
};

inline float Vector::Dot(const Vector &vector1)//, const Vector &vector2)
{
	return (vector1.x * x) + (vector1.y * y) + (vector1.z * z);
};

inline Vector::Vector(float x, float y, float z) : x(x), y(y), z(z) {}

inline Vector &Vector::operator +=(const Vector &vector)
{
	x += vector.x, y += vector.y, z += vector.z;
	return *this;
};

inline bool Vector::operator ==(const Vector &vector) const
{
	return ((x == vector.x) && (y == vector.y) && (z == vector.z));
};

inline bool Vector::operator !=(const Vector &vector) const
{
	return !(*this == vector);
};

inline Vector Vector::operator *(Vector v) const
{
	return Vector(x * v.x, y * v.y, z * v.z);
};

inline Vector &Vector::operator -=(const Vector &vector)
{
	x -= vector.x, y -= vector.y, z -= vector.z;
	return *this;
};
/*
inline float *Vector::operator [ ]( int row)
{
	return &m_v[row];
};
*/
inline Vector &Vector::operator *=(float f)
{
	x *= f, y *= f, z *= f;
	return *this;
};

inline Vector &Vector::operator /=(float f)
{
	x /= f, y /= f, z /= f;
	return *this;
};

inline Vector Vector::operator +(const Vector &vector) const
{
	Vector temp(*this);
	temp += vector;
	return temp;
};

inline Vector Vector::operator -(const Vector &vector) const
{
	Vector temp(*this);
	temp -= vector;
	return temp;
};

inline Vector Vector::operator *(float f) const
{
	return Vector(x * f,y * f, z * f);
};

inline Vector Vector::operator /(float f) const
{
	return Vector(x / f, y / f, z / f);
};
inline float Vector::Magnitude() const
{
	return sqrtf((x * x) + (y * y) + (z * z));
};
inline float Vector::MagnitudeSq() const
{
	return (x * x) + (y * y) + (z * z);
};
inline Vector Vector::Inverse() const
{
	return Vector(-x, -y, -z);
};
inline void Vector::Normalize()
{
	float InvertMag = 1.0f / Magnitude();
	x *= InvertMag, y *= InvertMag, z *= InvertMag;
};

inline bool Vector::operator <(const Vector &vector) const
{
	return ((x < vector.x) || (y < vector.y) || (z < vector.z));
};

inline bool Vector::operator >(const Vector &vector) const
{
	return ((x > vector.x) || (y > vector.y) || (z > vector.z));
};

inline float Vector::GetLegnth()
{
	return sqrt((x * x) + (y * y) + (z * z));
};

inline Vector Vector::operator /(Vector v) const
{
	return Vector(x / v.x, y / v.y, z / v.z);
};

inline float Vector::InvSqrt(float x)
{
	union
	{
		float f;
		int i;
	} u;

	float xhalf = 0.5f * x;

	u.f = x;
	u.i = 0x5f3759df - (u.i >> 1);
	x = u.f;
	x = x * (1.5f - xhalf * x * x);

	return x;
};

/*
inline void Vector::set(char *in)
{
	float x, y, z;

	sscanf(in, "%f %f %f", &x, &y, &z);

	// stop the compiler moaning :)
	this->x = x;
	this->y = y;
	this->z = z;
};*/

inline void Vector::set(float x, float y, float z)
{
	this->x = x, this->y = y, this->z = z;
};

#endif
