#include "draw.h"
#include "graphics.h"
#include "gmath.h"

void DrawLine ( const Vector &v1, const Vector &v2, Colour colour )
{
	glPushMatrix();
	{
		glEnable ( GL_BLEND );
		glMatrixMode ( GL_MODELVIEW );
		glLoadIdentity();
		glDisable ( GL_BLEND );
		glDisable ( GL_TEXTURE_2D );
// 		if(colour)
		glColor4fv ( colour.colour );

		glLineWidth ( 1.0f );

		glBegin ( GL_LINES );
		glVertex3f(v1.x, v1.y, v1.z);
		glVertex3f(v2.x, v2.y, v2.z);
		glEnd();


	}
	glPopMatrix();
};

void DrawLineRect(const Vector &v1,int width, int height, Colour colour)
{
			DrawLine(v1, Vector(v1.x + width, v1.y, v1.z), colour);
			DrawLine(Vector(v1.x + width, v1.y, v1.z), Vector(v1.x + width, v1.y + height, v1.z), colour);
			DrawLine(Vector(v1.x, v1.y + height, v1.z), Vector(v1.x + width, v1.y + height, v1.z), colour);
			DrawLine(Vector(v1.x, v1.y, v1.z), Vector(v1.x, v1.y + height, v1.z), colour);
};

void DrawRect(int x, int y, int width, int height, Colour colour)
{
	glPushMatrix();
	{
		glDisable(GL_BLEND);
		glDisable(GL_TEXTURE_2D);

		glTranslatef(x, y, 0);
// 		if(colour)
		glColor4fv(colour.colour);
		//printf("%f, %f, %f\n", colour.colour[0],colour.colour[1],colour.colour[2],colour.colour[3]);
		glBegin(GL_QUADS);
			glVertex2d(1, 1);
			glVertex2d(width, 1);
			glVertex2d(width, height);
			glVertex2d(1, height);
		glEnd();
	}
	glPopMatrix();
};

void DrawLineCirle(Vector pos, int radius, Colour colour)
{
	Vector temp, temp2;
	temp.x = pos.x;
	temp.y = pos.y + radius;
	glPushMatrix();
	glDisable(GL_TEXTURE_2D);
	//glDisable(GL_BLEND);
	glColor4fv(colour.colour);
	glLineWidth ( 1.0f );
	glBegin(GL_LINE_STRIP);
	{
		for(float angle = 0.0f; angle <= (2.0f * PI); angle += 0.01f)
		{
			temp2.x = pos.x + (radius * (float)sin((double)angle));
			temp2.y = pos.y + (radius * (float)cos((double)angle));
			glVertex3f(temp.x, temp.y, 0);
			temp.x = temp2.x;
			temp.y = temp2.y;
		}
	}
	glEnd();

	glPopMatrix();
};

void DrawCircle(Vector pos, int radius, Colour colour)
{
	Vector temp, temp2;
	temp.x = pos.y;
	temp.y = pos.x;
	glPushMatrix();
	glColor4fv(colour.colour);
	glBegin(GL_TRIANGLES);
	{
		for(int i = 0; i <= 360; i++)
		{
			float angle = (float)i / 57.29577957795135;
			temp2.x = temp.x + (radius * sin((double)angle));
			temp2.y = temp.y + (radius * cos((double)angle));
			glVertex3f(pos.x, pos.y, 0);
			glVertex3f(temp.x, temp.y, 0);
			glVertex3f(temp2.x, temp2.y, 0);
			temp = temp2;
		}
	}
	glEnd();

	glPopMatrix();
};

void DrawTex(const Vector &v1, float width, float height, const Texture2D &tex, Colour colour, bool aspect, float scale, float rotate)
{
	glPushMatrix();
	glLoadIdentity();
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	tex.Bind();
	glColor4fv(colour.colour);
	glTranslatef(v1.x, v1.y, 0);
	glScalef(scale, scale, 0);

	glRotatef(rotate, 0.0f, 0.0f, 1.0f);
	//glTranslatef((v1.x - (tex.GetWidth() / 2)), (v1.y - (tex.GetHeight() / 2)), 0);
	float x, y;
	x = (tex.GetWidth() / 2);
	y = (tex.GetHeight() / 2);
	glBegin(GL_QUADS);
	{
		glTexCoord2f(0.0, 0.0);
		glVertex2d(-x, -y);

		glTexCoord2f(1.0, 0.0);
		glVertex2d(x,-y);

		glTexCoord2f(1.0, 1.0);
		glVertex2d(x,y);

		glTexCoord2f(0.0, 1.0);
		glVertex2d(-x,y);
	}
	glEnd();
	glPopMatrix();
};

