#include "force.h"
#include "draw.h"
#include "input.h"
#include "collision.h"
#include "portal.h"
#include "texturemanager.h"

Force::Force(Vector pos, int radius, int Enum)
{
	m_position = pos;
	m_radius = radius;
	//m_type = Enum;
//	oldMouse_x = oldMouse_y = 0;
	m_force.set(0.0f, 0.0f, 0.0f);
	m_active = false;
	SetType(Enum);
};

Force::~Force()
{
};

void Force::SetType(int Enum)
{
	m_force.set(0,0,0);
	switch(Enum)
	{
		case fUp:
			m_force.y = 5.0f;
			m_type = Enum;
		break;
		case fDown:
			m_force.y = -5.0f;
			m_type = Enum;
		break;
		case fLeft:
			m_force.x = -5.0f;
			m_type = Enum;
		break;
		case fRight:
			m_force.x = 5.0f;
			m_type = Enum;
		break;
		default:
			m_force.y = 5.0f;
			m_type = fUp;
		break;
	}
	printf("passed type to force: %d\n", m_type);
};

void Force::SetActive(bool active)
{
	m_active = active;
};

void Force::Update()
{
	if(m_active == true)
	{
		int mouseX = Input::GetSingleton().GetX();
		int mouseY = 600 - Input::GetSingleton().GetY();
		int oldX = Input::GetSingleton().GetOldX();
		int oldY = Input::GetSingleton().GetOldY();
		if(Input::GetSingleton().mouseStillDown(SDL_BUTTON_LEFT) == true)
		{
			m_position.x = mouseX;
			m_position.y = mouseY;
		}
		else if(Input::GetSingleton().mouseStillDown(SDL_BUTTON_RIGHT) == true)
		{
			if(oldX != mouseX)
			{
				if(mouseX > oldX)
				{
					m_radius += (mouseX - oldX);
					if(m_radius > 100)
						m_radius = 100;
				}
				else if(mouseX < oldX)
				{
					m_radius -= (oldX - mouseX);
					if(m_radius < 20)
						m_radius = 20;
				}
			}
		}
		else if(Input::GetSingleton().mouseDown(SDL_BUTTON_WHEELDOWN) == true)
		{
			SetType(m_type + 1);
		}
		else if(Input::GetSingleton().mouseDown(SDL_BUTTON_WHEELUP) == true)
		{
			SetType(m_type - 1);
		}
	}
};
void Force::Render()
{
	DrawLineCirle(m_position, m_radius, Colour(255,255,255,255));
};

ForceManager::ForceManager()
{
	m_forces.clear();
	m_ActiveForce = -1;
	oldMouseX = oldMouseY = 0;
	m_dir = TextureManager::GetSingleton().Load("dir.png");
};

ForceManager::~ForceManager()
{
	ClearAllForces();
};

void ForceManager::AddForce(Vector pos, int radius, int type)
{
	Force *newForce = new Force(pos, radius, type);
	m_forces.push_back(newForce);
};

void ForceManager::ClearAllForces()
{
	for(uint i = 0; i < m_forces.size(); ++i)
		m_forces.erase(m_forces.begin() + i);

	m_forces.clear();
	m_ActiveForce = -1;
};

void ForceManager::Update()
{
	Vector mouse;
	mouse.x = Input::GetSingleton().GetX();
	mouse.y = 600 - Input::GetSingleton().GetY();

	mouse.z = 0;
	if(m_ActiveForce == -1)
	{
		for(uint i = 0; i < m_forces.size(); ++i)
		{
			//if(Input::GetSingleton().mouseStillDown(SDL_BUTTON_LEFT) == true ||
					//Input::GetSingleton().mouseStillDown(SDL_BUTTON_RIGHT) == true)
			//{
			if(InCircle(mouse, m_forces[i]->GetPosition(), m_forces[i]->GetRadius()) == true)
			{
				if(Input::GetSingleton().mouseDown(SDL_BUTTON_LEFT) == true ||
				   Input::GetSingleton().mouseDown(SDL_BUTTON_RIGHT) == true ||
				   Input::GetSingleton().mouseDown(SDL_BUTTON_WHEELDOWN) == true ||
				   Input::GetSingleton().mouseDown(SDL_BUTTON_WHEELUP) == true)
				{
					m_ActiveForce = (int)i;
					m_forces[i]->SetActive(true);
					m_active = true;
					break;
				}
			}
			//
			//}
		}
	}

	if(m_ActiveForce != -1)
	{
		if(InCircle(mouse, m_forces[m_ActiveForce]->GetPosition(), m_forces[m_ActiveForce]->GetRadius()) == true)
		{
			m_forces[m_ActiveForce]->Update();
		}
		else
		{
			m_forces[m_ActiveForce]->SetActive(false);
			m_ActiveForce = -1;
			m_active = false;
		}
	}
};

void ForceManager::Render()
{
	for(uint i = 0; i < m_forces.size(); ++i)
	{
		float rotate = 0;
		switch(m_forces[i]->GetType())
		{
			case fUp:
			break;
			case fDown:
				rotate = 180;
			break;
			case fRight:
				rotate = -90;
			break;
			case fLeft:
				rotate = 90;
			break;
		}
		m_forces[i]->Render();
		DrawTex(m_forces[i]->GetPosition(), 0,0, *m_dir, COLOUR_WHITE, true, 1, rotate);
	}
};
