#include "graphics.h"
#include <ctime>

Graphics::Graphics()
{
	m_SDL_flags = 0;
	m_bpp = 0;
	m_fullscreen = false;
	m_loaded = false;
	m_height = 0;
	m_width = 0;
	m_screen = NULL;
	m_windowName = "";
	m_renderMode = NO_MODE;
	printf("=================\nGraphics Init\n=================\n");
};

Graphics::~ Graphics()
{
	Shutdown();
};

bool Graphics::Init(int width, int height, int bpp, Uint32 SDL_flags, std::string WindowName)
{
	m_width = width;
	m_height = height;
	m_bpp = bpp;
	m_windowName = WindowName;
	m_SDL_flags = SDL_flags;

	if(m_loaded)
	{
		//gloabal reload
	}
	return true;
};

bool Graphics::MakeWindow()
{
	if(SDL_Init(SDL_INIT_VIDEO) != 0)
	{
		printf("Unable to Initialize SDL %s\n", SDL_GetError());
		exit(1);
	}
	const SDL_VideoInfo *videoInfo = SDL_GetVideoInfo();

	if(videoInfo->hw_available)
		m_SDL_flags |= SDL_HWSURFACE;
	else
		m_SDL_flags |= SDL_SWSURFACE;

	//checks if we can do hardware blits
	if(videoInfo->blit_hw)
		m_SDL_flags |= SDL_HWACCEL;

	// Sets up OpenGL double buffering
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

	atexit(SDL_Quit);

	m_screen = SDL_SetVideoMode(m_width, m_height, m_bpp, m_SDL_flags);

	if(m_screen == NULL)
	{
		printf("Failed to set Video Mode %s\n", SDL_GetError());
		exit(1);
	}

	InitGL();

	//Set Window Caption
	SDL_WM_SetCaption(m_windowName.c_str(), NULL);
	return true;
};

void Graphics::ResizeWindow(int width, int height)
{
	if ( height == 0 )
    	{
        	height = 1;
    	}

    	//resize sdl surface
	m_screen = SDL_SetVideoMode( width, height, m_bpp, m_SDL_flags );

	m_width = width;
	m_height = height;

	if (m_screen == NULL)
	{
		printf("Could not get a surface after resize: %s", SDL_GetError());
		exit(1);
	}

	InitGL();
};

int Graphics::GetWidth()
{
	return m_width;
};

int Graphics::GetHeight()
{
	return m_height;
};

Vector Graphics::convertToWorldCoords(GLfloat winX, GLfloat winY, Vector *near, Vector *far, Vector *dir)
{
	Vector result;

	GLdouble rx, ry, rz;
	GLdouble nx, ny, nz;
	GLdouble fx, fy, fz;

	GLfloat winZ;

	glGetIntegerv(GL_VIEWPORT, viewport);
	glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
	glGetDoublev(GL_PROJECTION_MATRIX, projection);

	winY = viewport[3] - winY;
	glReadPixels((int)winX, (int)winY, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &winZ);

	gluUnProject(winX, winY, winZ, modelview, projection, viewport, &rx, &ry, &rz);

	result.set(rx, ry , rz);
	if((near == NULL) || (far == NULL) || (dir == NULL))
		return result;

	gluUnProject(winX, winY, 0.0, modelview, projection, viewport, &nx, &ny, &nz);
	gluUnProject(winX, winY, 1.0, modelview, projection, viewport, &fx, &fy, &fz);

	near->set(nx, ny, nz);
	far->set(fx, fy, fz);

	*dir = *far;
	*dir -= *near;
	dir->Normalize();

	return result;
};

void Graphics::InitGL()
{
	glEnable( GL_TEXTURE_2D );
	glClearColor(0.0f,0.0f,0.0f,1.0f );
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glShadeModel(GL_SMOOTH);

	glClearDepth(1.0f);

	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	glDisable(GL_LIGHTING);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	glDepthFunc(GL_LEQUAL);

	GLenum error = glewInit();
	if(error != GLEW_OK)
		printf("failed to start GLEW\n");
	else
	{
		printf("GLEW started\n");
	}

};

void Graphics::ToggleFullscreen()
{
	m_fullscreen = !m_fullscreen;

	if(m_screen != NULL)
	{
		SDL_WM_ToggleFullScreen(m_screen);

		if(!m_fullscreen)
		{
			SDL_ShowCursor(SDL_ENABLE);
		}
		else
		{
			SDL_ShowCursor(SDL_DISABLE);
		}
	}
};

void Graphics::SwapBuffers()
{
	SDL_GL_SwapBuffers();
};

void Graphics::ClearScreen()
{
	glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
// 	glClearColor(1.0f,1.0f,1.0f,1.0f);
	glLoadIdentity();
};

void Graphics::Shutdown()
{
	//SDL_Quit();
	//not needed SDL_Quit() called atexit sweet
};

void Graphics::SetMode(int mode)
{
	switch(mode)
	{
		case ORTHOGRAPHIC:
			glPushMatrix();
			glViewport(0, 0, m_width, m_height);
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();

			glOrtho(0.0f, (GLdouble)m_width, 0.0f, (GLdouble)m_height, -1.0f, 1.0f);

			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();
	// 		glPushMatrix();
			glEnable(GL_SMOOTH);
			glEnable( GL_TEXTURE_2D );
			glEnable(GL_BLEND);
			glDisable(GL_LIGHTING);
			glDisable(GL_DEPTH_TEST);
			glLoadIdentity();
			m_renderMode = mode;
			break;
		case PROJECTION:
			glViewport(0, 0, m_width, m_height);

			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
	// 		glMultMatrixf(&App::camera.projectionMatrix[0][0]);
			gluPerspective(45.0f, (GLfloat)m_width / (GLfloat)m_height, 1.0f, 1250.0f);

			glMatrixMode(GL_MODELVIEW);
			glEnable(GL_SMOOTH);
			glEnable( GL_TEXTURE_2D );
// 			glEnable(GL_BLEND);
			glDisable(GL_LIGHTING);
			glLoadIdentity();
	// 		glMultMatrixf(&App::camera.viewMatrix[0][0]);
// 			glEnable(GL_DEPTH_TEST);
			m_renderMode = mode;
			break;
		case END_2D:
			glPopMatrix();
			m_renderMode = PROJECTION;
		default:
			break;
	}
};

GLfloat Graphics::GetRatio()
{
	return static_cast<GLfloat>(m_height / m_width);
};

int Graphics::GetMode()
{
	return m_renderMode;
};

int Graphics::ScreenShot()
{
	unsigned char *pixels;
	int hour, minute, sec, year;
	unsigned int hash = 0;
	char hashkey[20];
	std::string filename ="screenshot-";
	time(&date);
	char *buffer = ctime(&date);
	sscanf(buffer, "%*s %*s %*d %d:%d:%d %d", &hour, &minute, &sec, &year);
	sprintf(hashkey,"%d%d%d%d", year, hour, minute, sec);

	for(unsigned int i = 0; i < strlen(hashkey); i++)
	{
		hash = hash * 65599 + hashkey[i];
	}
//	sprintf(filename, "screenshot-%d.tga",hash);
	sprintf(hashkey,"%d", hash);
	FILE *file;
	int screenstats[4];
	filename.append(hashkey);
	filename.append(".tga");
	glGetIntegerv(GL_VIEWPORT, screenstats);

	pixels = new unsigned char[screenstats[2] * screenstats[3] * 3];
	glReadPixels(0, 0, screenstats[2], screenstats[3], GL_BGR, GL_UNSIGNED_BYTE, pixels);
	if((file=fopen(filename.c_str(), "wb")) == NULL)
		return 1;

	unsigned char TGAheader[12] = {0,0,2,0,0,0,0,0,0,0,0,0};
	unsigned char header[6] = {((int)(screenstats[2]%256)),
				((int)(screenstats[2]/256)),
				((int)(screenstats[3]%256)),
				((int)(screenstats[3]/256)),24,0};
	fwrite(TGAheader, sizeof(unsigned char), 12, file);
	fwrite(header, sizeof(unsigned char), 6, file);
	fwrite(pixels, sizeof(unsigned char), screenstats[2] * screenstats[3] * 3, file);

	fclose(file);
	delete [] pixels;
	return 0;
};
