#include "particle.h"
#include "graphics.h"
#include "draw.h"
#include "collision.h"

Particle::Particle()
{
	m_life = 0.0f;
	m_isAlive = false;
	m_size = 1.0f;
	m_weight = 0.0f;
};

Particle::~ Particle()
{
};

void Particle::Init(const Vector &pos, const Vector &vel, const Vector &accel, float life,
			float size, float weight, const Colour colour, int type)
{
	m_position = pos;
	m_velocity = vel;
	m_acceleration = accel;
	m_colour = colour;

	m_life = life;
	m_isAlive = true;
	m_size = size;
	m_weight = weight;
	m_type = type;
};

void Particle::Update(float timems)
{
	if(m_isAlive)
	{
		m_life -= timems;

		if(m_life <= 0.0f)
		{
			m_isAlive = false;
		}
		else
		{
			for(uint i = 0; i < ForceManager::GetSingleton().GetForcesSize(); ++i)
			{
				bool incircle = InCircle(m_position, ForceManager::GetSingleton().GetForces(i)->GetPosition(),
						ForceManager::GetSingleton().GetForces(i)->GetRadius());
				if(incircle == true)
				{
					m_velocity += ForceManager::GetSingleton().GetForces(i)->GetForce();
				}
			}

			for(uint i = 0; i < PortalManager::GetSingleton().GetPortalsSize(); ++i)
			{
				bool incircle = InCircle(m_position, PortalManager::GetSingleton().GetPortals(i)->GetPosition(),
						PortalManager::GetSingleton().GetPortals(i)->GetRadius());
				if(incircle == true)
				{
					m_type = PortalManager::GetSingleton().GetPortals(i)->GetType();
					m_colour = PortalManager::GetSingleton().GetPortals(i)->GetColour();
				}
			}

			m_velocity.x += m_acceleration.x * timems;
			m_velocity.y += m_acceleration.y * timems;
			m_velocity.z += m_acceleration.z * timems;

			m_position.x += m_velocity.x * timems;
			m_position.y += m_velocity.y * timems;
			m_position.z += m_velocity.z * timems;

			/*if(m_velocity.y < 0.0f)
			{
				if(m_position.y < (m_size * 0.5f))
 					m_velocity.y = -kFriction * m_velocity.y + ((rand() % 255) / 255.0f);
			}*/
		}

	}
};

void Particle::DrawPoint(bool reflect)
{
	if(reflect)
	{
		//do nothing
	}
	else
	{
		glColor4fv(m_colour.colour);
	}

	glVertex3f(m_position.x, m_position.y, m_position.z);

};
void Particle::DrawQuad(int halfX, int halfY)
{
	glColor4fv(m_colour.colour);
	glTexCoord2f(0.0, 0.0);
	glVertex2d(m_position.x - halfX, m_position.y - halfY);

	glTexCoord2f(1.0, 0.0);
	glVertex2d(m_position.x + halfX, m_position.y - halfY);

	glTexCoord2f(1.0, 1.0);
	glVertex2d(m_position.x + halfX, m_position.y + halfY);

	glTexCoord2f(0.0, 1.0);
	glVertex2d(m_position.x - halfX, m_position.y + halfY);

};
