/***************************************************************************
 *   Copyright (C) 2007 by Gavin Massey,,,   *
 *   mdk@griever   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PARTICLE_H_
#define PARTICLE_H_

#include "globals.h"
#include "cvector.h"
#include "colour.h"
#include "texture.h"
#include "force.h"
#include "portal.h"

// Gravity constant (in m/s^2)
const float kGravity = 9.81f;
// const float kGravity = 0.0f;
// Frictionnal force at bounce (1.0 = no friction,
// 0.0 = maximum friction)
const float kFriction = 0.5f;


class Particle
{
public:
	Particle();
	~Particle();

	void Init(const Vector &pos, const Vector &vel, const Vector &accel, float life,
			float size, float weight, const Colour colour, int type);
	void Update(float timems);
	void DrawPoint(bool reflect);
	void DrawQuad(int halfX, int halfY);
	void DrawLight();

	bool IsAlive() const { return m_isAlive; }
private:
	Vector m_position;
	Vector m_velocity;
	Vector m_acceleration;
	int m_type;
	float m_life;
	bool m_isAlive;

	float m_size;
	float m_weight;
	Colour m_colour;
};

#endif
