/***************************************************************************
 *   Copyright (C) 2007 by Gavin Massey,,,   *
 *   mdk@griever   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PARTICLE_SYSTEM_H_
#define PARTICLE_SYSTEM_H_

#include "globals.h"
#include "cvector.h"
#include "colour.h"
#include "timer.h"
#include "particle.h"
#include "texture.h"
//#include "quat.h"

class ParticleSystem
{
	friend class ParticleManager;
public:
	ParticleSystem(int maxParticles, float life, const Vector &origin);
	virtual ~ParticleSystem();

	virtual void Update(float timems) = 0;
	virtual void Render(bool reflect) = 0;
	virtual int Emit(float numParticles) = 0;
	virtual void LoadTexture(std::string filename) = 0;
	virtual void StartEmitting() { m_emitting = true; }
	virtual void StopEmitting() { m_emitting = false; }
protected:
	virtual void InitParticle(Particle &p) = 0;
	bool m_emitting;
	Particle *m_particleList;
	Vector m_origin;
	int m_maxParticles;
	int m_numParticles;
	float m_particleLife;
	float m_accumulatedTime;
	Texture2D *m_texture;
// 	Timer *m_timer;

};

#endif
