#include "portal.h"
#include "draw.h"
#include "input.h"
#include "collision.h"

Portal::Portal(Vector pos, int radius, int colour)
{
	m_position = pos;
	m_radius = radius;
	ChooseColour(colour);
	SetType(colour);
	m_active = false;
};

Portal::~Portal()
{
};

void Portal::SetType(int Enum)
{
	switch(Enum)
	{
		case RED:
			m_colour = COLOUR_RED;
			m_type = RED;
		break;
		case BLUE:
			m_colour = COLOUR_BLUE;
			m_type = BLUE;
		break;
		case CYAN:
			m_colour = COLOUR_CYAN;
			m_type = CYAN;
		break;
		case YELLOW:
			m_colour = COLOUR_YELLOW;
			m_type = YELLOW;
		break;
		case GREEN:
			m_colour = COLOUR_GREEN;
			m_type = GREEN;
		break;
		case PURPLE:
			m_colour = COLOUR_PURPLE;
			m_type = PURPLE;
		break;
		case WHITE:
			m_colour = COLOUR_WHITE;
			m_type = WHITE;
		break;
		default:
			m_colour = COLOUR_WHITE;
			m_type = WHITE;
		break;
	};
};

void Portal::Update()
{
	if(m_active == true)
	{
		int mouseX = Input::GetSingleton().GetX();
		int mouseY = 600 - Input::GetSingleton().GetY();
		int oldX = Input::GetSingleton().GetOldX();
		int oldY = Input::GetSingleton().GetOldY();
		if(Input::GetSingleton().mouseStillDown(SDL_BUTTON_LEFT) == true)
		{
			if(oldX != mouseX || oldY != mouseY)
			{
				m_position.x = mouseX;
				m_position.y = mouseY;
			}
		}
		else if(Input::GetSingleton().mouseStillDown(SDL_BUTTON_RIGHT) == true)
		{
			if(oldX != mouseX)
			{
				if(mouseX > oldX)
				{
					m_radius += (mouseX - oldX);
					if(m_radius > 100)
						m_radius = 100;
				}
				else if(mouseX < oldX)
				{
					m_radius -= (oldX - mouseX);
					if(m_radius < 20)
						m_radius = 20;
				}
			}
		}
		else if(Input::GetSingleton().mouseDown(SDL_BUTTON_WHEELDOWN) == true)
		{
			SetType(m_type + 1);
		}
		else if(Input::GetSingleton().mouseDown(SDL_BUTTON_WHEELUP) == true)
		{
			SetType(m_type - 1);
		}
	}
};

void Portal::Render()
{
	DrawLineCirle(m_position, m_radius, m_colour);
};

void Portal::ChooseColour(int colour)
{
	switch(colour)
	{
		case BLUE:
			m_colour = COLOUR_BLUE;
		break;
		case RED:
			m_colour = COLOUR_RED;
		break;
		case GREEN:
			m_colour = COLOUR_GREEN;
		break;
		case CYAN:
			m_colour = COLOUR_CYAN;
		break;
		case YELLOW:
			m_colour = COLOUR_YELLOW;
		break;
		case PURPLE:
			m_colour = COLOUR_PURPLE;
		break;
		case WHITE:
			m_colour = COLOUR_WHITE;
		break;
		default:
			printf("unsupported colour(WTF does that mean? :)\n");
	}
};


PortalManager::PortalManager()
{
	m_portals.clear();
	m_ActivePortal = -1;
	oldMouseX = oldMouseY = 0;
};

PortalManager::~PortalManager()
{
	ClearAllPortals();
};

void PortalManager::ClearAllPortals()
{
	for(uint i = 0; i < m_portals.size(); ++i)
		m_portals.erase(m_portals.begin() + i);

	m_portals.clear();
};

void PortalManager::AddPortal(Vector pos, int radius, int type)
{
	Portal *newPortal = new Portal(pos, radius, type);
	m_portals.push_back(newPortal);
};

void PortalManager::Update()
{
	Vector mouse;
	mouse.x = Input::GetSingleton().GetX();
	mouse.y = 600 - Input::GetSingleton().GetY();

	mouse.z = 0;
	if(m_ActivePortal == -1)
	{
		for(uint i = 0; i < m_portals.size(); ++i)
		{
			if(Input::GetSingleton().mouseStillDown(SDL_BUTTON_LEFT) == true ||
			   Input::GetSingleton().mouseStillDown(SDL_BUTTON_WHEELUP) == true ||
			   Input::GetSingleton().mouseStillDown(SDL_BUTTON_WHEELDOWN) == true ||
			   Input::GetSingleton().mouseStillDown(SDL_BUTTON_RIGHT) == true)
			{
				if(InCircle(mouse, m_portals[i]->GetPosition(), m_portals[i]->GetRadius()) == true)
				{
						m_portals[i]->SetActive(true);
						m_ActivePortal = (int)i;
						m_active = true;
						break;

				}
			}
		}
	}

	if(m_ActivePortal != -1)
	{
		if(InCircle(mouse, m_portals[m_ActivePortal]->GetPosition(), m_portals[m_ActivePortal]->GetRadius()) == true)
		{
			m_portals[m_ActivePortal]->Update();
		}
		else
		{
			m_portals[m_ActivePortal]->SetActive(false);
			m_ActivePortal = -1;
			m_active = false;
		}
	}
	oldMouseX = mouse.x;
	oldMouseY = mouse.y;
};

void PortalManager::Render()
{
	for(uint i = 0; i < m_portals.size(); ++i)
		m_portals[i]->Render();
};
