#ifndef PORTAL_H_
#define PORTAL_H_

#include "globals.h"
#include "cvector.h"
#include "colour.h"

enum COLOURS
{
	RED = 300,
	BLUE,
	CYAN,
	YELLOW,
	GREEN,
	PURPLE,
	WHITE,
};
#define COLOUR_WHITE Colour(255,255,255,255)
#define COLOUR_RED Colour(255,0,0,255)
#define COLOUR_GREEN Colour(0,255,0,255)
#define COLOUR_BLUE Colour(0,0,255,255)
#define COLOUR_CYAN Colour(0,255,255,255)
#define COLOUR_YELLOW Colour(255,255,0,255)
#define COLOUR_PURPLE Colour(127,0,255,255)

#define MAX_COLOURS 7

class Portal
{
public:
	Portal(Vector pos, int radius, int colour);
	~Portal();
	//dev purposes :)
	void Update();
	void Render();
	int GetRadius() const {return m_radius;}
	int GetType() const {return m_type;}
	Colour GetColour() const { return m_colour; }
	Vector GetPosition() const { return m_position; }
	bool isActive() const {return m_active; }
	void SetActive(bool active) { m_active = active; }
private:
	void SetType(int Enum);
	void ChooseColour(int colour); //misleading name :)
	Colour m_colour;
	Vector m_position;
	int m_radius;
	bool m_active;
	int m_type;
};

class PortalManager : public Singleton<PortalManager>
{
public:
	PortalManager();
	~PortalManager();

	void AddPortal(Vector pos, int radius, int colour);
	Portal *GetPortals(int index) { return m_portals[index]; }
	int GetPortalsSize() const { return m_portals.size(); }
	void Update();
	void Render();
	bool isActive() const { return m_active; }
	void ClearAllPortals();
private:
	std::vector<Portal *> m_portals;
	int m_ActivePortal;
	bool m_active;
	int oldMouseX, oldMouseY;
};

#endif // PORTAL_H_
