#ifndef TEXTURE_H_
#define TEXTURE_H_

#include "globals.h"
#include "image.h"

class Texture
{
public:
	Texture();
	virtual ~Texture();
	enum
	{
		kDefault = 0,
		kCompress = (1 << 0),
	};
	typedef int TextureFlags;

	void Bind() const;
	void Bind(GLenum texUnit) const;
	bool fail() const { return m_fail; }
	bool stdCoordSys() const { return m_standerdCoordSystem; }

	const std::string &name() const { return m_name; }
	const GLuint handle() const { return m_handle; }
	const int GetWidth() const {return m_width; }
	const int GetHeight() const {return m_height; }
	virtual GLenum target() const = 0;
private:
	Texture(const Texture &);
	Texture &operator =(const Texture &);
protected:
	GLubyte *LoadImageFile(const std::string &filename);
	GLint GetCompressionFormat(GLint internalFormat);
	GLint GetInternalFormat(GLint components);

	std::string m_name;
	GLuint m_handle;
	int m_width, m_height;
	TextureFlags m_flags;
	bool m_standerdCoordSystem;
	bool m_fail;

};

class Texture2D : public Texture
{
public:
	Texture2D(const std::string &filename, TextureFlags flags = kDefault);
	Texture2D(const Image *img, TextureFlags flags = kDefault);

	virtual GLenum target() const { return GL_TEXTURE_2D; }
protected:
	Texture2D();
	virtual void create(const Image *img, TextureFlags flags);
};

#endif
