#include "texturemanager.h"
// #include "graphictypes.h"
#include "singleton.h"
#include "log.h"

TextureManager::TextureManager()
{
};

TextureManager::~ TextureManager()
{
	DeleteAllTextures();
};

Texture2D *TextureManager::Load(const std::string &filename)
{
	Texture2D *tex = getTex(filename);

	if(tex == NULL)
	{
		tex = new Texture2D(filename);

		if(tex->fail() == true)
		{
			delete tex;
			tex = NULL;
		}
		else
		{
			RegisterTexture(tex->name(), tex);
		}
	}
	return tex;
};

Texture2D *TextureManager::Load(unsigned char *pixels)
{
	ImageBuffer *ibuff = new ImageBuffer((GLubyte *)pixels);
	Image *img = new Image(ibuff->filename(), 128, 128, GL_RGBA, 1, 0, ibuff->data(), true);
	Texture2D *tex = new Texture2D(img, Texture::kDefault);

	if(!tex->fail())
	{
		RegisterTexture(tex->name(), tex);
		delete ibuff;
	}
	else
	{
		delete tex;
		tex = NULL;
	}
	return tex;
};

Texture2D *TextureManager::getTex(const std::string &name)
{
	std::map<std::string, Texture2D *>::iterator itr;

	itr = m_texMap.find(name);

	if(itr != m_texMap.end())
		return itr->second;
	else
		return NULL;
};

void TextureManager::RegisterTexture(const std::string &name, Texture2D *tex)
	throw(std::runtime_error)
{
	std::pair<std::map<std::string, Texture2D *>::iterator, bool> res;

	res = m_texMap.insert(std::map<std::string, Texture2D *>::value_type(name, tex));

	if(!res.second)
	{
		throw std::runtime_error("Name Collision");
	}
	printf("Texture: %s loaded\n", tex->name().c_str());
	Log::GetSingleton().Write(LOG_APP, "Texture: %s loaded\n", tex->name().c_str());
};

void TextureManager::UnRegisterTexture(const std::string &name, bool deleteTex)
{
	std::map<std::string, Texture2D *>::iterator itr;
	itr = m_texMap.find(name);

	if(itr != m_texMap.end())
	{
		if(deleteTex)
			delete itr->second;

		m_texMap.erase(itr);

		printf("Texture: %s unloaded\n", name.c_str());
		Log::GetSingleton().Write(LOG_APP, "Texture: %s unloaded\n", name.c_str());
	}
};

void TextureManager::DeleteAllTextures()
{
	std::map<std::string, Texture2D *>::iterator itr = m_texMap.begin();
	for(itr; itr != m_texMap.end(); itr++)
	{
		delete itr->second;
	}

	m_texMap.clear();
};
